"""
Interfaces to ESGF Security APIs as defined in the `ESGF Security Interface`_

.. _`ESGF Security Interface`: https://github.com/ESGF/esgf.github.io/wiki/Security%7CInterfaceControlDocument

"""

NS = {
    'soap11': "http://schemas.xmlsoap.org/soap/envelope/",
    'saml2p': "urn:oasis:names:tc:SAML:2.0:protocol",
    'saml': "urn:oasis:names:tc:SAML:2.0:assertion",
}
