"""
Top-level package for Python interface to ESGF services.

"""

__version__ = '0.1.4b'

#!TODO: ResultFormatter class.  process response json to specialise the result json.  Default is None
#!TODO: pipe results to new process.  Command-line interface.
#!TODO: Helper methods for "get opendap" "get download urls"

    
    

    
